///////////////////////////////////////////
//
// file:  base_app.cpp
//
// Copyright 1999 Johnny Be Good! Software
//
//                John R. Ashmun
//                jbg@kagi.com
//
///////////////////////////////////////////

#include "base_app.h"

const char* signature = "application/x-vnd.jbg-Tracker_AddOn_Test_Base";

base_app::base_app( )
   : BApplication( signature )
{
}


base_app::~base_app( )
{
}


typedef void (*process_refs_func)( entry_ref, BMessage*, void* );


void base_app::RefsReceived( BMessage* msg )
{
   int32             number_found = 0;
   status_t          status = B_OK;
   image_id          id = 0;
   app_info          appInfo;
   type_code         type_found = 0;
   entry_ref         dir_ref;
   entry_ref         entry_in_dir_ref;
   BPath             add_on_path;
   BPath             add_ons_path;
   BPath             app_path;
   BPath             parent_path;
   BEntry            add_on_entry;
   BEntry*           app_entry = 0;
   BEntry            dir;
   BEntry*           entry_in_dir = 0;
   BDirectory*       add_ons_dir = 0;
   process_refs_func current_process_refs = 0;

   status = be_roster->GetAppInfo( signature, &appInfo );

   if( status != B_OK )
   {
      printf( "Couldn't get app_info for myself!\n" );
      goto quitpoint;
   }

   app_entry = new BEntry( &(appInfo.ref) );

   if( !app_entry )
   {
      printf( "Couldn't create BEntry from entry_ref to myself!\n" );
      goto quitpoint;
   }

   status = app_entry->InitCheck( );

   if( status != B_OK )
   {
      printf( "BEntry created from entry_ref to myself initialized incorrectly!\n" );
      delete app_entry;
      goto quitpoint;
   }

   status = app_entry->GetPath( &app_path );
   delete app_entry;

   if( status != B_OK )
   {
      printf( "Couldn't GetPath( ) to myself!\n" );
      goto quitpoint;
   }

   status = app_path.GetParent( &parent_path );

   if( status != B_OK )
   {
      printf( "Couldn't get my parent's path!\n" );
      goto quitpoint;
   }

   add_ons_path = parent_path;

   status = add_ons_path.Append( "add-ons" );

   if( status != B_OK )
   {
      printf( "Couldn't Append( \"add-ons\" ) to my parent's path!\n" );
      goto quitpoint;
   }

   add_ons_dir = new BDirectory( add_ons_path.Path( ) );

   if( !add_ons_dir )
   {
      printf( "Couldn't create BDirectory for add-ons directory!\n" );
      goto quitpoint;
   }

   status = add_ons_dir->InitCheck( );

   if( status != B_OK )
   {
      printf( "BDirectory created for add-ons directory initialized incorrectly!\n" );
      delete add_ons_dir;
      goto quitpoint;
   }

   status = add_ons_dir->GetNextEntry( &add_on_entry );
   delete add_ons_dir;

   if( status != B_OK )
   {
      printf( "Couldn't GetNextEntry( ) from add-ons directory!\n" );
      goto quitpoint;
   }

   status = add_on_entry.InitCheck( );

   if( status != B_OK )
   {
      printf( "BEntry for add-on initialized incorrectly!\n" );
      goto quitpoint;
   }

   status = add_on_entry.GetPath( &add_on_path );

   if( status != B_OK )
   {
      printf( "Couldn't GetPath( ) for add-on!\n" );
      goto quitpoint;
   }

//   printf( "Going to try to load %s\n", add_on_path.Leaf( ) );

   id = load_add_on( add_on_path.Leaf( ) );

   if( id == B_ERROR )
   {
      printf( "Couldn't load add-on!\n" );
      goto quitpoint;
   }

   status = get_image_symbol( id,
                              "process_refs",
                              B_SYMBOL_TYPE_TEXT,
                              (void**)&current_process_refs );

   if( status != B_OK )
   {
      printf( "Loaded add-on OK; couldn't find process_refs( )!\n" );
      goto bailout;
   }

//   printf( "Loaded add-on OK, found process_refs( ).\n" );

   status = msg->GetInfo( "refs", &type_found, &number_found );

   if( !number_found )
   {
      printf( "Got REFS_RECEIVED message with no refs in it!\n" );
      goto bailout;
   }

   status = msg->FindRef( "refs", 0, &entry_in_dir_ref );

   if( status != B_OK )
   {
      printf( "RefsReceived( ):  Couldn't find any entry_refs!\n" );
      goto bailout;
   }

   entry_in_dir = new BEntry( &entry_in_dir_ref );

   if( !entry_in_dir )
   {
      printf( "Couldn't create BEntry for add-on!\n" );
      goto bailout;
   }

   status = entry_in_dir->InitCheck( );

   if( status != B_OK )
   {
      printf( "BEntry created for add-on was initialized incorrectly!\n" );
      delete entry_in_dir;
      goto bailout;
   }

   status = entry_in_dir->GetParent( &dir );
   delete entry_in_dir;

   if( status != B_OK )
   {
      printf( "Found add-on's entry_ref and built entry from it,\n" );
      printf( "but couldn't get its parent!\n" );
      goto bailout;
   }

   status = dir.InitCheck( );

   if( status != B_OK )
   {
      printf( "Got the add-on's parent, but its InitCheck( ) failed!\n" );
      goto bailout;
   }

   status = dir.GetRef( &dir_ref );

   if( status != B_OK )
   {
      printf( "Couldn't GetRef( ) for add-ons directory!\n" );
      goto bailout;
   }

   current_process_refs( dir_ref, msg, 0 );

bailout:
   unload_add_on( id );

quitpoint:
   be_app->PostMessage( B_QUIT_REQUESTED );
}
